@@ -1,6 +1,8 @@
 /* radare2 - LGPL - Copyright 2013-2024 - pancake */
 
 #include <r_asm.h>
+#include <r_endian.h>
+#include "mips_utils.h"
 #include <capstone/capstone.h>
 #include <capstone/mips.h>
 
@@ -24,13 +26,13 @@ R_IPI int mips_assemble(const char *str, ut64 pc, ut8 *out);
 	}
 
 #define CREATE_SRC_DST_3(op) \
-	src0 = r_vector_push (&(op)->srcs, NULL);\
-	src1 = r_vector_push (&(op)->srcs, NULL);\
-	dst = r_vector_push (&(op)->dsts, NULL);
+	src0 = RVecRArchValue_emplace_back (&(op)->srcs);\
+	src1 = RVecRArchValue_emplace_back (&(op)->srcs);\
+	dst = RVecRArchValue_emplace_back (&(op)->dsts);
 
 #define CREATE_SRC_DST_2(op) \
-	src0 = r_vector_push (&(op)->srcs, NULL);\
-	dst = r_vector_push (&(op)->dsts, NULL);
+	src0 = RVecRArchValue_emplace_back (&(op)->srcs);\
+	dst = RVecRArchValue_emplace_back (&(op)->dsts);
 
 #define SET_SRC_DST_3_REGS(op) \
 	CREATE_SRC_DST_3 (op); \
@@ -59,27 +61,26 @@ R_IPI int mips_assemble(const char *str, ut64 pc, ut8 *out);
 // ESIL macros:
 
 // put the sign bit on the stack
-#define ES_IS_NEGATIVE(arg) "1,"arg",<<<,1,&"
+#define ES_IS_NEGATIVE(arg) "1,"arg",ROL,1,&"
 
 
-// call with delay slot
-#define ES_CALL_DR(ra, addr) "pc,4,+,"ra",=,"ES_J(addr)
+// Call with delay slot.
+#define ES_CALL_DR(ra, addr) "pc,4,+,"ra",=,"ES_J_D(addr)
 #define ES_CALL_D(addr) ES_CALL_DR("ra", addr)
 
-// call without delay slot
-#define ES_CALL_NDR(ra, addr) "pc,"ra",=,"ES_J(addr)
+// Call without delay slot.
+#define ES_CALL_NDR(ra, addr) "pc,"ra",=,"ES_J_ND(addr)
 #define ES_CALL_ND(addr) ES_CALL_NDR("ra", addr)
 
-#define USE_DS 0
-#if USE_DS
-// emit ERR trap if executed in a delay slot
-#define ES_TRAP_DS() "$ds,!,!,?{,$$,1,TRAP,BREAK,},"
-// jump to address
-#define ES_J(addr) addr",SETJT,1,SETD"
-#else
-#define ES_TRAP_DS() ""
-#define ES_J(addr) addr",pc,:="
-#endif
+// Delay-slot helper macros.
+// Trap if executed in a delay slot.
+#define ES_TRAP_DS(addr) "$ds,!,!,?{," addr ",1,TRAP,BREAK,},"
+// Jump with delay slot: record target and mark delay.
+#define ES_J_D(addr) addr",SETJT,1,SETD"
+// Jump without delay slot: update PC immediately.
+#define ES_J_ND(addr) addr",pc,:="
+// For likely branches: skip delay slot by advancing PC over it.
+#define ES_SKIP_NXT() "pc,4,+,pc,:="
 
 #define ES_B(x) "0xff,"x",&"
 #define ES_H(x) "0xffff,"x",&"
@@ -240,11 +241,13 @@ static const char *arg(csh *handle, cs_insn *insn, char *buf, size_t buf_sz, int
 static int analop_esil(RArchSession *as, RAnalOp *op, csh *handle, cs_insn *insn) {
 	char str[8][32] = {{0}};
 	int i;
+	ut64 addr = op->addr;
 
 	r_strbuf_init (&op->esil);
 	r_strbuf_set (&op->esil, "");
 
 	if (insn) {
+		addr = insn->address;
 		// caching operands
 		for (i = 0; i < insn->detail->mips.op_count && i < 8; i++) {
 			*str[i] = 0;
@@ -313,386 +316,403 @@ static int analop_esil(RArchSession *as, RAnalOp *op, csh *handle, cs_insn *insn
 		case MIPS_INS_SLL:
 			r_strbuf_appendf (&op->esil, "%s,%s,<<,%s,=", ARG (2), ARG (1), ARG (0));
 			break;
+		case MIPS_INS_BALC:
+			// BALC address
+			// Branch And Link, Compact. Unconditional PC relative branch to address,
+			// placing return address in register $31.
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_CALL_ND ("%s"), addr, ARG (0));
+			break;
 		case MIPS_INS_BAL:
 		case MIPS_INS_JAL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "" ES_CALL_D ("%s"), ARG (0));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_CALL_D ("%s"), addr, ARG (0));
 			break;
 		case MIPS_INS_JALR:
 		case MIPS_INS_JALRS:
 			if (OPCOUNT () < 2) {
-				r_strbuf_appendf (&op->esil, ES_TRAP_DS () "" ES_CALL_D ("%s"), ARG (0));
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_CALL_D ("%s"), addr, ARG (0));
 			} else {
 				PROTECT_ZERO () {
-					r_strbuf_appendf (&op->esil, ES_TRAP_DS () "" ES_CALL_DR ("%s", "%s"), ARG (0), ARG (1));
+					r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_CALL_DR ("%s", "%s"), addr, ARG (0), ARG (1));
 				}
 			}
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
 			break;
 		case MIPS_INS_JALRC: // no delay
 			if (OPCOUNT () < 2) {
-				r_strbuf_appendf (&op->esil, ES_TRAP_DS () "" ES_CALL_ND ("%s"), ARG (0));
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_CALL_ND ("%s"), addr, ARG (0));
 			} else {
 				PROTECT_ZERO () {
-					r_strbuf_appendf (&op->esil, ES_TRAP_DS () "" ES_CALL_NDR ("%s", "%s"), ARG (0), ARG (1));
+					r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_CALL_NDR ("%s", "%s"), addr, ARG (0), ARG (1));
 				}
 			}
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
 			break;
 		case MIPS_INS_JRADDIUSP:
 			// increment stackpointer in X and jump to %ra
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "%s,sp,+=," ES_J ("ra"), ARG (0));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,sp,+=," ES_J_D ("ra"), addr, ARG (0));
+			break;
+		case MIPS_INS_JRC:
+		case MIPS_INS_BC:
+			// JRC rt
+			// Jump Register, Compact. Unconditional jump to address in register $rt.
+			// BC address
+			// Branch, Compact. Unconditional PC relative branch to address.
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_J_ND ("%s"), addr, ARG (0));
 			break;
 		case MIPS_INS_JR:
-		case MIPS_INS_JRC:
 		case MIPS_INS_J:
 		case MIPS_INS_B: // ???
 			// jump to address with conditional
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "" ES_J ("%s"), ARG (0));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "" ES_J_D ("%s"), addr, ARG (0));
 			break;
+		case MIPS_INS_BNEC:
+				// BNEC rs, rt, address
+				// Branch Not Equal, Compact. PC relative branch to address if register $rs is not equal to
+				// register $rt.
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,%s,^,?{," ES_J_ND ("%s") ",}",
+					addr, ARG (0), ARG (1), ARG (2));
+				break;
 		case MIPS_INS_BNE: // bne $s, $t, offset
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,%s,^,?{," ES_J_D ("%s") ",}",
+					addr, ARG (0), ARG (1), ARG (2));
+				break;
 		case MIPS_INS_BNEL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "%s,%s,==,$z,!,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1), ARG (2));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				// BNEL rs, rt, offset
+				// To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if
+				// the branch is taken.
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,%s,^,?{," ES_J_D ("%s") ",}{," ES_SKIP_NXT () ",}",
+					addr, ARG (0), ARG (1), ARG (2));
+				break;
+		case MIPS_INS_BEQC:
+				// BEQC rs, rt, address
+				// Branch if Equal, Compact. PC relative branch to address if registers $rs and $rt are are equal.
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,%s,^,!,?{," ES_J_ND ("%s") ",}",
+					addr, ARG (0), ARG (1), ARG (2));
+				break;
 		case MIPS_INS_BEQ:
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,%s,^,!,?{," ES_J_D ("%s") ",}",
+					addr, ARG (0), ARG (1), ARG (2));
+				break;
 		case MIPS_INS_BEQL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "%s,%s,==,$z,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1), ARG (2));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				// BEQL rs, rt, offset
+				// To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if
+				// the branch is taken.
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,%s,^,!,?{," ES_J_D ("%s") ",}{," ES_SKIP_NXT () ",}",
+					addr, ARG (0), ARG (1), ARG (2));
+				break;
+		case MIPS_INS_BEQZC:
+				// BEQZC rt, address # when rt and address are in range
+				// Branch if Equal to Zero, Compact. PC relative branch to address if register $rt equals zero.
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,!,?{," ES_J_ND ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BZ:
 		case MIPS_INS_BEQZ:
-		case MIPS_INS_BEQZC:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "%s,0,==,$z,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,!,?{," ES_J_D ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
+		case MIPS_INS_BNEZC:
+				// BNEZC rt, address
+				// Branch if Not Equal to Zero, Compact. PC relative branch to address if register $rt is not equal to zero.
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,?{," ES_J_ND ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BNEZ:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "%s,0,==,$z,!,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,?{," ES_J_D ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BEQZALC:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "%s,0,==,$z,?{," ES_CALL_ND ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,!,?{," ES_CALL_ND ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
+		case MIPS_INS_BLEZC:
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "0,%s,<=,?{," ES_J_ND ("%s") ",},",
+				addr, ARG (0), ARG (1));
 			break;
 		case MIPS_INS_BLEZ:
-		case MIPS_INS_BLEZC:
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "0,%s,<=,?{," ES_J_D ("%s") ",},",
+				addr, ARG (0), ARG (1));
+			break;
 		case MIPS_INS_BLEZL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0,%s,==,$z,?{," ES_J ("%s") ",BREAK,},",
-				ARG (0), ARG (1));
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "1," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "0,%s,<=,?{," ES_J_D ("%s") ",}{," ES_SKIP_NXT () ",}",
+				addr, ARG (0), ARG (1));
 			break;
-		case MIPS_INS_BGEZ:
+		case MIPS_INS_BGEC:
+				// BGEC rs, rt, address
+				// Branch if Greater than or Equal, Compact. PC relative branch to address if register $rs
+				// is greater than or equal to register $rt.
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "%s,%s,>=,?{," ES_J_ND ("%s") ",}",
+					addr, ARG (1), ARG (0), ARG (2));
+				break;
 		case MIPS_INS_BGEZC:
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",!,?{," ES_J_ND ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
+		case MIPS_INS_BGEZ:
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",!,?{," ES_J_D ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BGEZL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",!,?{," ES_J_D ("%s") ",}{," ES_SKIP_NXT () ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BGEZAL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_CALL_D ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",!,?{," ES_CALL_D ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BGEZALC:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_CALL_ND ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",!,?{," ES_CALL_ND ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BGTZALC:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0,%s,==,$z,?{,BREAK,},", ARG (0));
-			r_strbuf_appendf (&op->esil, "0," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_CALL_ND ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "0,%s,>,?{," ES_CALL_ND ("%s") ",}",
+					addr, ARG (0), ARG (1));
 			break;
 		case MIPS_INS_BLTZAL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "1," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_CALL_D ("%s") ",}", ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
-		case MIPS_INS_BLTZ:
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",?{," ES_CALL_D ("%s") ",}",
+						addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BLTZC:
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",?{," ES_J_ND ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
+		case MIPS_INS_BLTZ:
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",?{," ES_J_D ("%s") ",}",
+					addr, ARG (0), ARG (1));
+				break;
 		case MIPS_INS_BLTZL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "1," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) ES_IS_NEGATIVE ("%s") ",?{," ES_J_D ("%s") ",}{," ES_SKIP_NXT () ",}",
+					addr, ARG (0), ARG (1));
+				break;
+		case MIPS_INS_BGTZC:
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "0,%s,>,?{," ES_J_ND ("%s") ",},",
+					addr, ARG (0), ARG (1));
 			break;
 		case MIPS_INS_BGTZ:
-		case MIPS_INS_BGTZC:
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "0,%s,>,?{," ES_J_D ("%s") ",},",
+					addr, ARG (0), ARG (1));
+			break;
 		case MIPS_INS_BGTZL:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0,%s,==,$z,?{,BREAK,},", ARG (0));
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0," ES_IS_NEGATIVE ("%s") ",==,$z,?{," ES_J ("%s") ",}",
-				ARG (0), ARG (1));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
+			r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "0,%s,>,?{," ES_J_D ("%s") ",}{," ES_SKIP_NXT () ",}",
+					addr, ARG (0), ARG (1));
 			break;
 		case MIPS_INS_BTEQZ:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0,t,==,$z,?{," ES_J ("%s") ",}", ARG (0));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "t,!,?{," ES_J_D ("%s") ",}", addr, ARG (0));
+				break;
 		case MIPS_INS_BTNEZ:
-			r_strbuf_appendf (&op->esil, ES_TRAP_DS () "0,t,==,$z,!,?{," ES_J ("%s") ",}", ARG (0));
-#if USE_DS
-			r_strbuf_replacef (&op->esil, "$$", "0x%"PFMT64x, addr);
-#endif
-			break;
-		case MIPS_INS_MOV:
-		case MIPS_INS_MOVE:
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "%s,%s,=", ARG (1), REG (0));
-			}
-			break;
-		case MIPS_INS_MOVZ:
-		case MIPS_INS_MOVF:
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "0,%s,==,$z,?{,%s,%s,=,}",
-					ARG (2), ARG (1), REG (0));
-			}
-			break;
-		case MIPS_INS_MOVN:
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "0,%s,==,$z,!,?{,%s,%s,=,}",
-					ARG (2), ARG (1), REG (0));
-			}
-			break;
-		case MIPS_INS_MOVT:
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "1,%s,==,$z,?{,%s,%s,=,}",
-					ARG (2), ARG (1), REG (0));
-			}
-			break;
-		case MIPS_INS_FSUB:
-		case MIPS_INS_SUB:
-		case MIPS_INS_SUBU:
-		case MIPS_INS_DSUB:
-		case MIPS_INS_DSUBU:
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "%s,%s,-,%s,=",
-					ARG (2), ARG (1), ARG (0));
-			}
-			break;
-		case MIPS_INS_NEG:
-		case MIPS_INS_NEGU:
-			r_strbuf_appendf (&op->esil, "%s,0,-,%s,=,",
-				ARG (1), ARG (0));
-			break;
+				r_strbuf_appendf (&op->esil, ES_TRAP_DS ("0x%"PFMT64x) "t,?{," ES_J_D ("%s") ",}", addr, ARG (0));
+				break;
+			case MIPS_INS_MOV:
+			case MIPS_INS_MOVE:
+				PROTECT_ZERO () {
+					r_strbuf_appendf (&op->esil, "%s,%s,=", ARG (1), REG (0));
+				}
+				break;
+			case MIPS_INS_MOVZ:
+			case MIPS_INS_MOVF:
+				PROTECT_ZERO () {
+					r_strbuf_appendf (&op->esil, "0,%s,==,$z,?{,%s,%s,=,}",
+						ARG (2), ARG (1), REG (0));
+				}
+				break;
+			case MIPS_INS_MOVN:
+				PROTECT_ZERO () {
+					r_strbuf_appendf (&op->esil, "0,%s,==,$z,!,?{,%s,%s,=,}",
+						ARG (2), ARG (1), REG (0));
+				}
+				break;
+			case MIPS_INS_MOVT:
+				PROTECT_ZERO () {
+					r_strbuf_appendf (&op->esil, "1,%s,==,$z,?{,%s,%s,=,}",
+						ARG (2), ARG (1), REG (0));
+				}
+				break;
+			case MIPS_INS_FSUB:
+			case MIPS_INS_SUB:
+			case MIPS_INS_SUBU:
+			case MIPS_INS_DSUB:
+			case MIPS_INS_DSUBU:
+				PROTECT_ZERO () {
+					r_strbuf_appendf (&op->esil, "%s,%s,-,%s,=",
+						ARG (2), ARG (1), ARG (0));
+				}
+				break;
+			case MIPS_INS_NEG:
+			case MIPS_INS_NEGU:
+				r_strbuf_appendf (&op->esil, "%s,0,-,%s,=,",
+					ARG (1), ARG (0));
+				break;
 
-		/** signed -- sets overflow flag */
-		case MIPS_INS_ADD:
-			{
+			/** signed -- sets overflow flag */
+			case MIPS_INS_ADD:
+				{
+					PROTECT_ZERO () {
+						ES_ADD_CK32_OVERF (ARG(1), ARG(2), ARG(0));
+					}
+				}
+				break;
+			case MIPS_INS_ADDI:
 				PROTECT_ZERO () {
 					ES_ADD_CK32_OVERF (ARG(1), ARG(2), ARG(0));
 				}
-			}
-			break;
-		case MIPS_INS_ADDI:
-			PROTECT_ZERO () {
-				ES_ADD_CK32_OVERF (ARG(1), ARG(2), ARG(0));
-			}
-			break;
-		case MIPS_INS_DADD:
-		case MIPS_INS_DADDI:
-			ES_ADD_CK64_OVERF (ARG(1), ARG(2), ARG(0));
-			break;
-		/** unsigned */
-		case MIPS_INS_DADDU:
-		case MIPS_INS_ADDU:
-		case MIPS_INS_ADDIU:
-		case MIPS_INS_DADDIU:
-			{
+				break;
+			case MIPS_INS_DADD:
+			case MIPS_INS_DADDI:
+				ES_ADD_CK64_OVERF (ARG(1), ARG(2), ARG(0));
+				break;
+			/** unsigned */
+			case MIPS_INS_DADDU:
+			case MIPS_INS_ADDU:
+			case MIPS_INS_ADDIU:
+			case MIPS_INS_DADDIU:
+				{
+					const char *arg0 = ARG(0);
+					const char *arg1 = ARG(1);
+					const char *arg2 = ARG(2);
+					PROTECT_ZERO () {
+						if (*arg2 == '-') {
+							r_strbuf_appendf (&op->esil, "%s,%s,-,%s,=",
+								arg2+1, arg1, arg0);
+						} else {
+							r_strbuf_appendf (&op->esil, "%s,%s,+,%s,=",
+								arg2, arg1, arg0);
+						}
+					}
+				}
+				break;
+			case MIPS_INS_LI:
+			case MIPS_INS_LDI:
+				r_strbuf_appendf (&op->esil, "0x%" PFMT64x ",%s,=", (ut64)IMM(1), ARG(0));
+				break;
+			case MIPS_INS_LUI:
+				r_strbuf_appendf (&op->esil, "0x%" PFMT64x "0000,%s,=", (ut64)IMM(1), ARG(0));
+				break;
+			case MIPS_INS_LB:
+				op->sign = true;
+				ESIL_LOAD ("1");
+				break;
+			case MIPS_INS_LBU:
+				//one of these is wrong
+				ESIL_LOAD ("1");
+				break;
+			case MIPS_INS_LW:
+			case MIPS_INS_LWC1:
+			case MIPS_INS_LWC2:
+			case MIPS_INS_LWL:
+			case MIPS_INS_LWR:
+			case MIPS_INS_LWU:
+			case MIPS_INS_LL:
+				ESIL_LOAD ("4");
+				break;
+
+			case MIPS_INS_LDL:
+			case MIPS_INS_LDC1:
+			case MIPS_INS_LDC2:
+			case MIPS_INS_LLD:
+			case MIPS_INS_LD:
+				ESIL_LOAD ("8");
+				break;
+
+			case MIPS_INS_LWX:
+			case MIPS_INS_LH:
+			case MIPS_INS_LHU:
+			case MIPS_INS_LHX:
+				ESIL_LOAD ("2");
+				break;
+
+			case MIPS_INS_AND:
+			case MIPS_INS_ANDI:
+				{
+					const char *arg0 = ARG(0);
+					const char *arg1 = ARG(1);
+					const char *arg2 = ARG(2);
+					if (!strcmp (arg0, arg1)) {
+						r_strbuf_appendf (&op->esil, "%s,%s,&=", arg2, arg1);
+					} else {
+						r_strbuf_appendf (&op->esil, "%s,%s,&,%s,=", arg2, arg1, arg0);
+					}
+				}
+				break;
+			case MIPS_INS_OR:
+			case MIPS_INS_ORI:
+				{
 				const char *arg0 = ARG(0);
 				const char *arg1 = ARG(1);
 				const char *arg2 = ARG(2);
 				PROTECT_ZERO () {
-					if (*arg2 == '-') {
-						r_strbuf_appendf (&op->esil, "%s,%s,-,%s,=",
-							arg2+1, arg1, arg0);
-					} else {
-						r_strbuf_appendf (&op->esil, "%s,%s,+,%s,=",
+					r_strbuf_appendf (&op->esil, "%s,%s,|,%s,=",
+						arg2, arg1, arg0);
+					}
+				}
+				break;
+			case MIPS_INS_XOR:
+			case MIPS_INS_XORI:
+				{
+					const char *arg0 = ARG(0);
+					const char *arg1 = ARG(1);
+					const char *arg2 = ARG(2);
+					PROTECT_ZERO () {
+						r_strbuf_appendf (&op->esil, "%s,%s,^,%s,=",
 							arg2, arg1, arg0);
 					}
 				}
-			}
-			break;
-		case MIPS_INS_LI:
-		case MIPS_INS_LDI:
-			r_strbuf_appendf (&op->esil, "0x%" PFMT64x ",%s,=", (ut64)IMM(1), ARG(0));
-			break;
-		case MIPS_INS_LUI:
-			r_strbuf_appendf (&op->esil, "0x%" PFMT64x "0000,%s,=", (ut64)IMM(1), ARG(0));
-			break;
-		case MIPS_INS_LB:
-			op->sign = true;
-			ESIL_LOAD ("1");
-			break;
-		case MIPS_INS_LBU:
-			//one of these is wrong
-			ESIL_LOAD ("1");
-			break;
-		case MIPS_INS_LW:
-		case MIPS_INS_LWC1:
-		case MIPS_INS_LWC2:
-		case MIPS_INS_LWL:
-		case MIPS_INS_LWR:
-		case MIPS_INS_LWU:
-		case MIPS_INS_LL:
-			ESIL_LOAD ("4");
-			break;
-
-		case MIPS_INS_LDL:
-		case MIPS_INS_LDC1:
-		case MIPS_INS_LDC2:
-		case MIPS_INS_LLD:
-		case MIPS_INS_LD:
-			ESIL_LOAD ("8");
-			break;
-
-		case MIPS_INS_LWX:
-		case MIPS_INS_LH:
-		case MIPS_INS_LHU:
-		case MIPS_INS_LHX:
-			ESIL_LOAD ("2");
-			break;
-
-		case MIPS_INS_AND:
-		case MIPS_INS_ANDI:
-			{
-				const char *arg0 = ARG(0);
-				const char *arg1 = ARG(1);
-				const char *arg2 = ARG(2);
-				if (!strcmp (arg0, arg1)) {
-					r_strbuf_appendf (&op->esil, "%s,%s,&=", arg2, arg1);
+				break;
+			case MIPS_INS_NOR:
+				{
+					const char *arg0 = ARG(0);
+					const char *arg1 = ARG(1);
+					const char *arg2 = ARG(2);
+					PROTECT_ZERO () {
+						r_strbuf_appendf (&op->esil, "%s,%s,|,0xffffffffffffffff,^,%s,=",
+							arg2, arg1, arg0);
+					}
+				}
+				break;
+			case MIPS_INS_SLT:
+			case MIPS_INS_SLTI:
+				if (OPCOUNT () < 3) {
+					r_strbuf_appendf (&op->esil, "%s,%s,<,t,=", ARG(1), ARG(0));
 				} else {
-					r_strbuf_appendf (&op->esil, "%s,%s,&,%s,=", arg2, arg1, arg0);
+					r_strbuf_appendf (&op->esil, "%s,%s,<,%s,=", ARG(2), ARG(1), ARG(0));
 				}
-			}
-			break;
-		case MIPS_INS_OR:
-		case MIPS_INS_ORI:
-			{
-			const char *arg0 = ARG(0);
-			const char *arg1 = ARG(1);
-			const char *arg2 = ARG(2);
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "%s,%s,|,%s,=",
-					arg2, arg1, arg0);
+				break;
+			case MIPS_INS_SLTU:
+			case MIPS_INS_SLTIU:
+				if (OPCOUNT () < 3) {
+					r_strbuf_appendf (&op->esil, ES_W("%s")","ES_W("%s")",<,t,=",
+						ARG (1), ARG (0));
+				} else {
+					r_strbuf_appendf (&op->esil, ES_W("%s")","ES_W("%s")",<,%s,=",
+						ARG (2), ARG (1), ARG (0));
 				}
-			}
-			break;
-		case MIPS_INS_XOR:
-		case MIPS_INS_XORI:
-			{
-				const char *arg0 = ARG(0);
-				const char *arg1 = ARG(1);
-				const char *arg2 = ARG(2);
+				break;
+			case MIPS_INS_MUL:
+				r_strbuf_appendf (&op->esil, ES_W("%s,%s,*")",%s,=", ARG(1), ARG(2), ARG(0));
+				ES_SIGN32_64 (ARG(0));
+				break;
+			case MIPS_INS_MULT:
+			case MIPS_INS_MULTU:
+				r_strbuf_appendf (&op->esil, ES_W("%s,%s,*")",lo,=", ARG (0), ARG (1));
+				ES_SIGN32_64 ("lo");
+				r_strbuf_appendf (&op->esil, ES_W("32,%s,%s,*,>>")",hi,=", ARG (0), ARG (1));
+				ES_SIGN32_64 ("hi");
+				break;
+			case MIPS_INS_MFLO:
 				PROTECT_ZERO () {
-					r_strbuf_appendf (&op->esil, "%s,%s,^,%s,=",
-						arg2, arg1, arg0);
+					r_strbuf_appendf (&op->esil, "lo,%s,=", REG (0));
 				}
-			}
-			break;
-		case MIPS_INS_NOR:
-			{
-				const char *arg0 = ARG(0);
-				const char *arg1 = ARG(1);
-				const char *arg2 = ARG(2);
+				break;
+			case MIPS_INS_MFHI:
 				PROTECT_ZERO () {
-					r_strbuf_appendf (&op->esil, "%s,%s,|,0xffffffffffffffff,^,%s,=",
-						arg2, arg1, arg0);
+					r_strbuf_appendf (&op->esil, "hi,%s,=", REG (0));
 				}
-			}
-			break;
-		case MIPS_INS_SLT:
-		case MIPS_INS_SLTI:
-			if (OPCOUNT () < 3) {
-				r_strbuf_appendf (&op->esil, "%s,%s,<,t,=", ARG(1), ARG(0));
-			} else {
-				r_strbuf_appendf (&op->esil, "%s,%s,<,%s,=", ARG(2), ARG(1), ARG(0));
-			}
-			break;
-		case MIPS_INS_SLTU:
-		case MIPS_INS_SLTIU:
-			if (OPCOUNT () < 3) {
-				r_strbuf_appendf (&op->esil, ES_W("%s")","ES_W("%s")",<,t,=",
-					ARG (1), ARG (0));
-			} else {
-				r_strbuf_appendf (&op->esil, ES_W("%s")","ES_W("%s")",<,%s,=",
-					ARG (2), ARG (1), ARG (0));
-			}
-			break;
-		case MIPS_INS_MUL:
-			r_strbuf_appendf (&op->esil, ES_W("%s,%s,*")",%s,=", ARG(1), ARG(2), ARG(0));
-			ES_SIGN32_64 (ARG(0));
-			break;
-		case MIPS_INS_MULT:
-		case MIPS_INS_MULTU:
-			r_strbuf_appendf (&op->esil, ES_W("%s,%s,*")",lo,=", ARG (0), ARG (1));
-			ES_SIGN32_64 ("lo");
-			r_strbuf_appendf (&op->esil, ES_W("32,%s,%s,*,>>")",hi,=", ARG (0), ARG (1));
-			ES_SIGN32_64 ("hi");
-			break;
-		case MIPS_INS_MFLO:
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "lo,%s,=", REG (0));
-			}
-			break;
-		case MIPS_INS_MFHI:
-			PROTECT_ZERO () {
-				r_strbuf_appendf (&op->esil, "hi,%s,=", REG (0));
-			}
-			break;
-		case MIPS_INS_MTLO:
-			r_strbuf_appendf (&op->esil, "%s,lo,=", REG (0));
-			ES_SIGN32_64 ("lo");
-			break;
-		case MIPS_INS_MTHI:
-			r_strbuf_appendf (&op->esil, "%s,hi,=", REG (0));
-			ES_SIGN32_64 ("hi");
-			break;
+				break;
+			case MIPS_INS_MTLO:
+				r_strbuf_appendf (&op->esil, "%s,lo,=", REG (0));
+				ES_SIGN32_64 ("lo");
+				break;
+			case MIPS_INS_MTHI:
+				r_strbuf_appendf (&op->esil, "%s,hi,=", REG (0));
+				ES_SIGN32_64 ("hi");
+				break;
 #if 0
 	// could not test div
 	case MIPS_INS_DIV:
@@ -731,14 +751,14 @@ static void op_fillval(RArchSession *as, RAnalOp *op, csh *handle, cs_insn *insn
 	switch (op->type & R_ANAL_OP_TYPE_MASK) {
 	case R_ANAL_OP_TYPE_LOAD:
 		if (OPERAND(1).type == MIPS_OP_MEM) {
-			src0 = r_vector_push (&op->srcs, NULL);
+			src0 = RVecRArchValue_emplace_back (&op->srcs);
 			src0->reg = parse_reg_name (*handle, insn, 1);
 			src0->delta = OPERAND(1).mem.disp;
 		}
 		break;
 	case R_ANAL_OP_TYPE_STORE:
 		if (OPERAND(1).type == MIPS_OP_MEM) {
-			dst = r_vector_push (&op->dsts, NULL);
+			dst = RVecRArchValue_emplace_back (&op->dsts);
 			dst->reg = parse_reg_name (*handle, insn, 1);
 			dst->delta = OPERAND(1).mem.disp;
 		}
@@ -988,31 +1008,26 @@ static bool decode(RArchSession *as, RAnalOp *op, RArchDecodeMask mask) {
 	case MIPS_INS_LDL:
 	case MIPS_INS_LDR:
 	case MIPS_INS_LDXC1:
-		op->type = R_ANAL_OP_TYPE_LOAD;
-		if (!op->refptr) {
-			op->refptr = 8;
-		}
-		switch (OPERAND(1).type) {
-		case MIPS_OP_MEM:
-			if (OPERAND(1).mem.base == MIPS_REG_GP) {
-				op->ptr = as->config->gp + OPERAND(1).mem.disp;
-#if 0
-				if (REGID (0) == MIPS_REG_T9) {
-					pd->t9_pre = op->ptr;
-					// read pointer again
-					ut32 na = 0;
-					RBin *bin = as->arch->binb.bin;
-					if (bin && bin->iob.read_at (bin->iob.io, op->ptr, &na, sizeof (na))) {
-						pd->t9_pre = na; // UT64_MAX;
-					}
-
-					// eprintf ("SET PRE9 0x%llx\n", op->ptr);
-				}
-#endif
-			} else if (REGID (0) == MIPS_REG_T9) {
-				pd->t9_pre = UT64_MAX;
+			op->type = R_ANAL_OP_TYPE_LOAD;
+			if (!op->refptr) {
+				op->refptr = 8;
 			}
-			break;
+			switch (OPERAND(1).type) {
+			case MIPS_OP_MEM:
+				if (OPERAND(1).mem.base == MIPS_REG_GP) {
+					op->ptr = as->config->gp + OPERAND(1).mem.disp;
+					if (REGID (0) == MIPS_REG_T9) {
+						pd->t9_pre = op->ptr;
+						RBin *bin = as->arch->binb.bin;
+						const ut64 ptrv = mips_read_ptr_at (bin, op->ptr, R_ARCH_CONFIG_IS_BIG_ENDIAN (as->config), as->config->bits);
+						if (ptrv != UT64_MAX) {
+							pd->t9_pre = ptrv;
+						}
+					}
+				} else if (REGID (0) == MIPS_REG_T9) {
+					pd->t9_pre = UT64_MAX;
+				}
+				break;
 		case MIPS_OP_IMM:
 			op->ptr = OPERAND(1).imm;
 			break;
@@ -1180,14 +1195,18 @@ static bool decode(RArchSession *as, RAnalOp *op, RArchDecodeMask mask) {
 	case MIPS_INS_B:
 	case MIPS_INS_BZ:
 	case MIPS_INS_BEQ:
+	case MIPS_INS_BEQC:
+	case MIPS_INS_BEQZ:
+	case MIPS_INS_BEQZC:
 	case MIPS_INS_BNZ:
 	case MIPS_INS_BNE:
+	case MIPS_INS_BNEC:
 	case MIPS_INS_BNEL:
 	case MIPS_INS_BEQL:
-	case MIPS_INS_BEQZ:
+	case MIPS_INS_BNEZ:
+	case MIPS_INS_BNEZC:
 	case MIPS_INS_BNEG:
 	case MIPS_INS_BNEGI:
-	case MIPS_INS_BNEZ:
 	case MIPS_INS_BTEQZ:
 	case MIPS_INS_BTNEZ:
 	case MIPS_INS_BLTZ:
@@ -1217,6 +1236,10 @@ static bool decode(RArchSession *as, RAnalOp *op, RArchDecodeMask mask) {
 		}
 
 		switch (insn->id) {
+		case MIPS_INS_BEQC:
+		case MIPS_INS_BEQZC:
+		case MIPS_INS_BNEC:
+		case MIPS_INS_BNEZC:
 		case MIPS_INS_BLEZC:
 		case MIPS_INS_BGEZC:
 		case MIPS_INS_BLTZC:
