@@ -86,7 +86,7 @@ static RCoreHelpMessage help_msg_at_at = {
 	"x", " @@f", "run 'x' on all functions (see aflq)",
 	"x", " @@f:write", "run 'x' on all functions matching write in the name",
 	"x", " @@F", "alias for @@c:afla - inverse recursive function list",
-	"x", " @@i", "run 'x' on all instructions of the current function (see pdr)",
+	"x", " @@i", "run 'x' on all instructions of current function (see pdr)",
 	"x", " @@iS", "run 'x' on all sections adjusting blocksize",
 	"x", " @@k sdbquery", "run 'x' on all offsets returned by that sdbquery",
 	"x", " @@s:from to step", "run 'x' on all offsets from, to incrementing by step",
@@ -117,7 +117,7 @@ static RCoreHelpMessage help_msg_at_at_at = {
 	"x", " @@@f:hit*", "flags matching glob expression",
 	"x", " @@@i", "imports",
 	"x", " @@@M", "dbg.maps (See ?$?~size)",
-	"x", " @@@m", "io.maps",
+	"x", " @@@m[:rwx]", "io.maps (all of the ones matching perm mask)",
 	"x", " @@@r", "registers",
 	"x", " @@@R", "relocs",
 	"x", " @@@S", "sections",
@@ -206,7 +206,7 @@ static RCoreHelpMessage help_msg_root = {
 	".", "[?] [-|(m)|f|!sh|cmd]", "Define macro or load r2, cparse or rlang file",
 	",", "[?] [/jhr]", "create and query or filter a table with data from file",
 	":", "cmd", "run an io command (same as =!)",
-	"-", "[?]", "open editor and run the r2 commands in the saved document",
+	"-", "[?]", "open editor and run the r2 commands saved in the file",
 	"_", "[?]", "Print last output",
 	"=", "[?] [cmd]", "submit or listen to remote commands",
 	"<", "[str]", "feed stdin with given escaped string",
@@ -264,6 +264,17 @@ static RCoreHelpMessage help_msg_question_i = {
 	NULL
 };
 
+static RCoreHelpMessage help_msg_clippy = {
+	"Usage: ?E[0-9KOC] msg", "print a message with clippy avatar", "",
+	"?E", " msg", "print message with default avatar (see scr.clippy)",
+	"?E0", " msg", "print message with first frame of avatar",
+	"?E1", " msg", "print message with second frame of avatar",
+	"?EK", " msg", "print message with cybercat avatar",
+	"?EO", " msg", "print message with orangg avatar",
+	"?EC", " msg", "print message with croco avatar",
+	NULL
+};
+
 static RCoreHelpMessage help_msg_question_e = {
 	"Usage: ?e[=bdgnpst] arg", "print/echo things", "",
 	"?e", "", "echo message with newline",
@@ -286,16 +297,13 @@ static RCoreHelpMessage help_msg_question_e = {
 
 static RCoreHelpMessage help_msg_question = {
 	"Usage: ?[?[?]] expression", "", "",
-	"?!", " [cmd]", "run cmd if $? == 0",
-	"?", " eip-0x804800", "show all representation result for this math expr",
+	"?_", " hudfile", "load hud menu with given file",
+	"??", "", "show help for ? commands",
 	"?'", "", "show help for the single quote (do not evaluate special characters in command)",
-	"?$", "", "show value all the variables ($)",
-	"?+", " [cmd]", "run cmd if $? > 0",
-	"?-", " [cmd]", "run cmd if $? < 0",
+	"?", " eip-0x804800", "show all representation result for this math expr",
 	"?=", " eip-0x804800", "update $? return code with result of operation",
 	"?==", " x86 `e asm.arch`", "strcmp two strings",
-	"??", " [cmd]", "run cmd if $? != 0",
-	"??", "", "show value of operation",
+	"?$", "", "show value all the variables ($)",
 	"?a", "", "show ascii table",
 	"?B", " [elem]", "show range boundaries like 'e?search.in",
 	"?b", " [num]", "show binary value of number",
@@ -303,15 +311,19 @@ static RCoreHelpMessage help_msg_question = {
 	"?btw", " num|expr num|expr num|expr", "returns boolean value of a <= b <= c",
 	"?d", " [num]", "disasssemble given number as a little and big endian dword",
 	"?e", "[=bdgnpst] arg", "echo messages, bars, pie charts and more (see ?e? for details)",
+	"?eq", " [cmd]", "run cmd if $? == 0",
 	"?f", " [num] [str]", "map each bit of the number as flag string index",
 	"?F", "", "flush cons output",
+	"?ge", " [cmd]", "run cmd if $? > 0",
 	"?h", " [str]", "calculate hash for given string",
 	"?i", "[?] arg", "prompt for number or Yes,No,Msg,Key,Path and store in $$?",
 	"?j", " arg", "same as '? num' but in JSON",
 	"?l", "[q] str", "returns the length of string ('q' for quiet, just set $?)",
+	"?le", " [cmd]", "run cmd if $? < 0",
+	"?ne", " [cmd]", "run cmd if $? != 0",
 	"?o", " num", "get octal value",
 	"?P", " paddr", "get virtual address for given physical one",
-	"?p", " vaddr", "get physical address for given virtual address",
+	"?p", " vaddr", "get physical address for given virtual one",
 	"?q", " num|expr", "compute expression like ? or ?v but in quiet mode",
 	"?r", " [from] [to]", "generate random number between from-to",
 	"?s", " from to step", "sequence of numbers from to by steps",
@@ -319,15 +331,14 @@ static RCoreHelpMessage help_msg_question = {
 	"?T", "", "show loading times",
 	"?u", " num", "get value in human units (KB, MB, GB, TB)",
 	"?v", " num|expr", "show hex value of math expr (no expr prints $?)",
+	"?V", "", "show library version of r_core",
 	"?vi", "[1248] num|expr", "show decimal value of math expr [n bytes]",
 	"?vx", " num|expr", "show 8 digit padding in hex",
-	"?V", "", "show library version of r_core",
 	"?w", " addr", "show what's in this address (like pxr/pxq does)",
 	"?X", " num|expr", "returns the hexadecimal value numeric expr",
 	"?x", " str", "returns the hexpair of number or string",
-	"?x", "+num", "like ?v, but in hexpairs honoring cfg.bigendian",
 	"?x", "-hexst", "convert hexpair into raw string with newline",
-	"?_", " hudfile", "load hud menu with given file",
+	"?x", "+num", "like ?v, but in hexpairs honoring cfg.bigendian",
 	"[cmd]?*", "", "recursive help for the given cmd",
 	NULL
 };
@@ -406,7 +417,7 @@ static RCoreHelpMessage help_msg_question_v = {
 	"$p", "", "getpid()",
 	"$P", "", "pid of children (only in debug)",
 	"$r", "", "get console height (in rows, see $c for columns)",
-	"$r", "{reg}", "get value of named register ($r{PC} and $r:PC syntax is supported)",
+	"$r", "{reg}", "get value of register <reg> ($r{PC} and $r:PC are supported)",
 	"$s", "", "file size",
 	"$w", "", "get word size, 4 if asm.bits=32, 8 if 64, ...",
 	"RNum", "", "$variables usable in math expressions",
@@ -465,9 +476,10 @@ static const char* findBreakChar(const char *s) {
 
 // XXX This is an experimental test and must be implemented in RCons directly
 static void colormessage(RCore *core, const char *msg) {
+	char padstr[128];
 	size_t msglen = strlen (msg);
 	RCons *cons = core->cons;
-	const char *pad = r_str_pad (' ', msglen + 5);
+	const char *pad = r_str_pad (padstr, sizeof (padstr), ' ', msglen + 5);
 	r_cons_gotoxy (cons, 10, 10); r_cons_printf (cons, Color_BGBLUE"%s", pad);
 	r_cons_gotoxy (cons, 10, 11); r_cons_printf (cons, Color_BGBLUE"%s", pad);
 	r_cons_gotoxy (cons, 10, 12); r_cons_printf (cons, Color_BGBLUE"%s", pad);
@@ -521,150 +533,8 @@ static char *filterFlags(RCore *core, const char *msg) {
 	return buf;
 }
 
-static const char *avatar_orangg[] = {
-	"      _______\n"
-	"     /       \\      .-%s-.\n"
-	"   _| ( o) (o)\\_    | %s |\n"
-	"  / _     .\\. | \\  <| %s |\n"
-	"  \\| \\   ____ / 7`  | %s |\n"
-	"  '|\\|  `---'/      `-%s-'\n"
-	"     | /----. \\\n"
-	"     | \\___/  |___\n"
-	"     `-----'`-----'\n"
-};
-
-static const char *avatar_croco[] = {
-	" __   __          .-%s-.\n"
-	"(o |_| o)_____    | %s |\n"
-	"|  ___________)  <  %s |\n"
-	"\\            /    | %s |\n"
-	" \\          /     `-%s-'\n"
-	"  \\________/\n"
-};
-
-static const char *avatar_clippy[] = {
-	" .--.     .-%s-.\n"
-	" | _|     | %s |\n"
-	" | O O   <  %s |\n"
-	" |  |  |  | %s |\n"
-	" || | /   `-%s-'\n"
-	" |`-'|\n"
-	" `---'\n",
-	" .--.     .-%s-.\n"
-	" |   \\    | %s |\n"
-	" | O o   <  %s |\n"
-	" |   | /  | %s |\n"
-	" |  ( /   `-%s-'\n"
-	" |   / \n"
-	" `--'\n",
-	" .--.     .-%s-.\n"
-	" | _|_    | %s |\n"
-	" | O O   <  %s |\n"
-	" |  ||    | %s |\n"
-	" | _:|    `-%s-'\n"
-	" |   |\n"
-	" `---'\n",
-};
-
-static const char *avatar_clippy_utf8[] = {
-	" ╭──╮    ╭─%s─╮\n"
-	" │ _│    │ %s │\n"
-	" │ O O  <  %s │\n"
-	" │  │╭   │ %s │\n"
-	" ││ ││   ╰─%s─╯\n"
-	" │└─┘│\n"
-	" ╰───╯\n",
-	" ╭──╮    ╭─%s─╮\n"
-	" │ ╶│╶   │ %s │\n"
-	" │ O o  <  %s │\n"
-	" │  │  ╱ │ %s │\n"
-	" │ ╭┘ ╱  ╰─%s─╯\n"
-	" │ ╰ ╱\n"
-	" ╰──'\n",
-	" ╭──╮    ╭─%s─╮\n"
-	" │ _│_   │ %s │\n"
-	" │ O O  <  %s │\n"
-	" │  │╷   │ %s │\n"
-	" │  ││   ╰─%s─╯\n"
-	" │ ─╯│\n"
-	" ╰───╯\n",
-};
-
-static const char *avatar_cybcat[] = {
-"     /\\.---./\\       .-%s-.\n"
-" '--           --'   | %s |\n"
-"----   ^   ^   ---- <  %s |\n"
-"  _.-    Y    -._    | %s |\n"
-"                     `-%s-'\n",
-"     /\\.---./\\       .-%s-.\n"
-" '--   @   @   --'   | %s |\n"
-"----     Y     ---- <  %s |\n"
-"  _.-    O    -._    | %s |\n"
-"                     `-%s-'\n",
-"     /\\.---./\\       .-%s-.\n"
-" '--   =   =   --'   | %s |\n"
-"----     Y     ---- <  %s |\n"
-"  _.-    U    -._    | %s |\n"
-"                     `-%s-'\n",
-};
-
-enum {
-	R_AVATAR_ORANGG,
-	R_AVATAR_CYBCAT,
-	R_AVATAR_CROCO,
-	R_AVATAR_CLIPPY,
-};
-
-R_API void r_core_clippy(RCore *core, const char *msg) {
-	int type = R_AVATAR_CLIPPY;
-	switch (*msg) {
-	case '+':
-	case '3':
-	case 'C':
-		{
-			const char *space = strchr (msg, ' ');
-			if (!space) {
-				space = msg;
-			}
-			type = (*msg == '+')? R_AVATAR_ORANGG: (*msg == 'C')? R_AVATAR_CROCO: R_AVATAR_CYBCAT;
-			msg = space + 1;
-		}
-		break;
-	}
-	const char *f;
-	int msglen = r_str_len_utf8 (msg);
-	char *s = strdup (r_str_pad (' ', msglen));
-	char *l;
-
-	switch (type) {
-	case R_AVATAR_ORANGG:
-		l = strdup (r_str_pad ('-', msglen));
-		f = avatar_orangg[0];
-		break;
-	case R_AVATAR_CROCO:
-		l = strdup (r_str_pad ('-', msglen));
-		f = avatar_croco[0];
-		break;
-	case R_AVATAR_CYBCAT:
-		l = strdup (r_str_pad ('-', msglen));
-		f = avatar_cybcat[r_num_rand (R_ARRAY_SIZE (avatar_cybcat))];
-		break;
-	default:
-		if (r_config_get_b (core->config, "scr.utf8")) {
-			l = (char *)r_str_repeat ("─", msglen);
-			f = avatar_clippy_utf8[r_num_rand (R_ARRAY_SIZE (avatar_clippy_utf8))];
-		} else {
-			l = strdup (r_str_pad ('-', msglen));
-			f = avatar_clippy[r_num_rand (R_ARRAY_SIZE (avatar_clippy))];
-		}
-		break;
-	}
-
-	r_cons_printf (core->cons, f, l, s, msg, s, l);
-	free (l);
-	free (s);
-}
 
+#include "clippy.inc.c"
 #include "visual_riu.inc.c"
 
 const char iuhelp[] =
@@ -1186,6 +1056,7 @@ static int cmd_help(void *data, const char *input) {
 			r_core_cmd_help (core, help_msg_question_v);
 		} else {
 			int i = 0;
+			char padstr[128];
 			const char *vars[] = {
 				"$$", "$$c", "$$$", "$$$c", "$?", "$b", "$c", "$Cn", "$D", "$DB", "$DD", "$Dn",
 				"$is", "$ij", "$if", "$ir", "$iv", "$in", "$ip",
@@ -1200,16 +1071,36 @@ static int cmd_help(void *data, const char *input) {
 			};
 			const bool wideOffsets = r_config_get_i (core->config, "scr.wideoff");
 			while (vars[i]) {
-				const char *pad = r_str_pad (' ', 6 - strlen (vars[i]));
+				const char *pad = r_str_pad (padstr, sizeof (padstr), ' ', 6 - strlen (vars[i]));
 				if (wideOffsets) {
-					eprintf ("%s %s 0x%016"PFMT64x"\n", vars[i], pad, r_num_math (core->num, vars[i]));
+					r_cons_printf (core->cons, "%s %s 0x%016"PFMT64x"\n", vars[i], pad, r_num_math (core->num, vars[i]));
 				} else {
-					eprintf ("%s %s 0x%08"PFMT64x"\n", vars[i], pad, r_num_math (core->num, vars[i]));
+					r_cons_printf (core->cons, "%s %s 0x%08"PFMT64x"\n", vars[i], pad, r_num_math (core->num, vars[i]));
 				}
 				i++;
 			}
 		}
 		return true;
+	case 'g':
+		if (input[1] == 'e') { // "?ge"
+			const char *cmd = r_str_trim_head_ro (input + 3);
+			if (*cmd && (st64)core->num->value > 0) {
+				r_core_cmd (core, cmd, 0);
+			}
+		} else {
+			r_core_return_invalid_command (core, "?g", input[1]);
+		}
+		break;
+	case 'n':
+		if (input[1] == 'e') { // "?ne"
+			const char *cmd = r_str_trim_head_ro (input + 3);
+			if (*cmd && core->num->value) {
+				r_core_cmd (core, cmd, 0);
+			}
+		} else {
+			r_core_return_invalid_command (core, "?n", input[1]);
+		}
+		break;
 	case 'V': // "?V"
 		switch (input[1]) {
 		case '?': // "?V?"
@@ -1234,6 +1125,7 @@ static int cmd_help(void *data, const char *input) {
 				pj_ks (pj, "arch", R_SYS_ARCH);
 				pj_ks (pj, "os", R_SYS_OS);
 				pj_ki (pj, "bits", R_SYS_BITS);
+				pj_ki (pj, "abiversion", R2_ABIVERSION);
 				pj_ki (pj, "commit", R2_VERSION_COMMIT);
 				pj_ks (pj, "tap", R2_GITTAP);
 				pj_ki (pj, "major", R2_VERSION_MAJOR);
@@ -1268,9 +1160,16 @@ static int cmd_help(void *data, const char *input) {
 		}
 		break;
 	case 'l': // "?l"
-		if (input[1] == 'q') {
-			for (input += 2; input[0] == ' '; input++);
+		if (input[1] == 'e') { // "?le"
+			const char *cmd = r_str_trim_head_ro (input + 3);
+			if (*cmd && (st64)core->num->value < 0) {
+				r_core_cmd (core, cmd, 0);
+			}
+		} else if (input[1] == 'q') {
+			input = r_str_trim_head_ro (input + 2);
 			r_core_return_value (core, strlen (input));
+		} else if (input[1] == '?') {
+			r_core_cmd_help_contains (core, help_msg_question, "?l");
 		} else {
 			input = r_str_trim_head_ro (input + 1);
 			r_core_return_value (core, strlen (input));
@@ -1312,9 +1211,22 @@ static int cmd_help(void *data, const char *input) {
 		}
 		break;
 	case 'E': // "?E" clippy echo
-		r_core_clippy (core, input + 1);
+		if (input[1] == '?') {
+			r_core_cmd_help (core, help_msg_clippy);
+		} else {
+			r_core_clippy (core, input + 1);
+		}
 		break;
 	case 'e': // "?e" echo
+		if (input[1] == 'q') { // "?eq"
+			const char *cmd = r_str_trim_head_ro (input + 3);
+			if (*cmd && core->num->value == 0) {
+				r_core_cmd (core, cmd, 0);
+			} else {
+				r_core_return_value (core, core->num->value);
+			}
+			break;
+		}
 		if (input[1] == ' ' && (input[2] == '"' || input[2] == '\'')) {
 			r_str_trim_args ((char *)input);
 		}
@@ -1397,13 +1309,13 @@ static int cmd_help(void *data, const char *input) {
 				int len = strlen (text) + 2;
 				RStrBuf *b = r_strbuf_new ("");
 				r_strbuf_append (b, ".");
-				r_strbuf_append (b, r_str_pad ('-', len));
+				r_strbuf_pad (b, '-', len);
 				r_strbuf_append (b, ".\n");
 				r_strbuf_append (b, "| ");
 				r_strbuf_append (b, text);
 				r_strbuf_append (b, " |\n");
 				r_strbuf_append (b, "'");
-				r_strbuf_append (b, r_str_pad ('-', len));
+				r_strbuf_pad (b, '-', len);
 				r_strbuf_append (b, "'\n");
 				char * s = r_strbuf_drain (b);
 				r_cons_print (core->cons, s);
@@ -1429,11 +1341,11 @@ static int cmd_help(void *data, const char *input) {
 						  char *d = r_str_donut (i);
 						  r_cons_gotoxy (core->cons, 0, 0);
 						  r_str_trim_tail (d);
-						  r_cons_clear_line (core->cons, 0);
+						  r_cons_clear_line (core->cons, false, true);
 						  r_cons_printf (core->cons, "Downloading the Gibson...\n\n");
 						  r_core_cmdf (core, "?e=%d", i);
 						  r_cons_print (core->cons, d);
-						  r_cons_clear_line (core->cons, 0);
+						  r_cons_clear_line (core->cons, false, true);
 						  r_cons_newline (core->cons);
 						  free (d);
 						  r_cons_flush (core->cons);
@@ -1531,17 +1443,20 @@ static int cmd_help(void *data, const char *input) {
 				  r_list_free (llist);
 			  }
 			break;
-		case ' ': {
-			const char *msg = r_str_trim_head_ro (input + 1);
-			// TODO: replace all ${flagname} by its value in hexa
-			char *newmsg = filterFlags (core, msg);
-			r_str_unescape (newmsg);
-			r_cons_println (core->cons, newmsg);
-			free (newmsg);
+		case ' ':
+			{
+				const char *msg = r_str_trim_head_ro (input + 1);
+				// TODO: replace all ${flagname} by its value in hexa
+				char *newmsg = filterFlags (core, msg);
+				r_str_unescape (newmsg);
+				r_cons_println (core->cons, newmsg);
+				free (newmsg);
+				r_core_return_value (core, 0);
 			}
 			break;
 		case 0: // "?e"
 			r_cons_newline (core->cons);
+			r_core_return_value (core, 0);
 			break;
 		case '?': // "?e?"
 			r_core_cmd_help (core, help_msg_question_e);
@@ -1693,22 +1608,13 @@ static int cmd_help(void *data, const char *input) {
 		}
 		break;
 	case '?': // "??"
-		if (input[1] == '?') {
-			if (input[2] == '?') { // "???"
-				r_core_clippy (core, "What are you doing?");
-				return 0;
-			}
-			if (input[2]) {
-				if (core->num->value) {
-					r_core_cmd (core, input + 1, 0);
-				}
-				break;
-			}
+		if (input[1] == 0) { // "??"
 			r_core_cmd_help (core, help_msg_question);
 			return 0;
 		} else if (input[1]) {
 			if (core->num->value) {
-				r_core_return_value (core, r_core_cmd (core, input + 1, 0));
+				r_core_cmd (core, input + 1, 0);
+				//r_core_return_value (core, rc);
 			}
 		} else {
 			if (core->num->dbz) {
@@ -1775,15 +1681,25 @@ static void cmd_head(void *data, const char *_input) { // "head"
 
 static int cmd_h(void *data, const char *_input) { // "head"
 	RCore *core = (RCore *)data;
-	if (r_str_startswith (_input, "ead")) {
-		cmd_head (data, _input);
-		return 0;
+	if (_input[0] == 'e') {
+		if (r_str_startswith (_input, "ead")) {
+			cmd_head (data, _input);
+			return 0;
+		}
+		if (r_str_startswith (_input, "elp")) {
+			r_cons_printf (core->cons, "%s\n", help_message);
+			return 0;
+		}
+		if (_input[1] == '?') {
+			r_core_cmd_help ((RCore*)data, help_msg_h);
+		} else {
+			r_core_return_invalid_command (core, "he", _input[1]);
+		}
+	} else if (_input[0] == '?') {
+		r_core_cmd_help ((RCore*)data, help_msg_h);
+	} else {
+		r_core_return_invalid_command (core, "h", _input[0]);
 	}
-	if (r_str_startswith (_input, "elp")) {
-		r_cons_printf (core->cons, "%s\n", help_message);
-		return 0;
-	}
-	r_core_cmd_help ((RCore*)data, help_msg_h);
 	return 0; // invalid command
 }
 #endif
