@@ -15,14 +15,34 @@ static const char *const rwxstr[] = {
 	[6] = "rw-",
 	[7] = "rwx",
 
-	[8] = "---",
-	[9] = "--x",
-	[10] = "-w-",
-	[11] = "-wx",
-	[12] = "r--",
-	[13] = "r-x",
-	[14] = "rw-",
-	[15] = "rwx",
+	[8] = "s---",
+	[9] = "s--x",
+	[10] = "sw--",
+	[11] = "sw-x",
+	[12] = "sr--",
+	[13] = "sr-x",
+	[14] = "srw-",
+	[15] = "srwx",
+};
+
+static const char *const srwxstr[] = {
+	[0] = "----",
+	[1] = "---x",
+	[2] = "--w-",
+	[3] = "--wx",
+	[4] = "-r--",
+	[5] = "-r-x",
+	[6] = "-rw-",
+	[7] = "-rwx",
+
+	[8] = "s---",
+	[9] = "s--x",
+	[10] = "sw--",
+	[11] = "sw-x",
+	[12] = "sr--",
+	[13] = "sr-x",
+	[14] = "srw-",
+	[15] = "srwx",
 };
 
 // equal string, same case
@@ -208,11 +228,12 @@ R_API int r_str_rwx(const char *str) {
 	R_RETURN_VAL_IF_FAIL (str, -1);
 	int ret = atoi (str);
 	if (!ret) {
+		ret |= strchr (str, 's')? 8: 0;
 		ret |= strchr (str, 'm')? 16: 0;
 		ret |= strchr (str, 'r')? 4: 0;
 		ret |= strchr (str, 'w')? 2: 0;
 		ret |= strchr (str, 'x')? 1: 0;
-		if (!ret && strcmp (str, "---")) {
+		if (!ret && strcmp (str, "---") && strcmp (str, "s---")) {
 			ret = -1;
 		}
 	} else if (ret < 0 || ret >= R_ARRAY_SIZE (rwxstr)) {
@@ -221,12 +242,26 @@ R_API int r_str_rwx(const char *str) {
 	return ret;
 }
 
-// Returns the string representation of the permission of the inputted integer.
-R_API const char *r_str_rwx_i(int rwx) {
-	if (rwx < 0 || rwx >= R_ARRAY_SIZE (rwxstr)) {
-		rwx = 0;
+// Returns the string representation of permissions.
+// with_dash=false: 3-4 chars "rwx" "s--x" (compact)
+// with_dash=true:  4 chars "-rwx" "s--x" (Unix-style)
+// If SHAR bit is set, returns 4 characters with 's' prefix instead of dash.
+R_API const char *r_str_perm(int perm, bool with_dash) {
+	if (perm < 0 || perm >= 16) {
+		return with_dash ? srwxstr[0] : rwxstr[0];
 	}
-	return rwxstr[rwx % 24]; // 15 for srwx
+	int idx = (perm & R_PERM_RWX) | ((perm & R_PERM_SHAR) ? 8 : 0);
+	return with_dash ? srwxstr[idx] : rwxstr[idx];
+}
+
+// Deprecated: use r_str_perm(perm, false) instead
+R_API const char *r_str_rwx_i(int rwx) {
+	return r_str_perm (rwx, false);
+}
+
+// Deprecated: use r_str_perm(perm, true) instead
+R_API const char *r_str_srwx_i(int rwx) {
+	return r_str_perm (rwx, true);
 }
 
 // If up is true, upcase all characters in the string, otherwise downcase all
@@ -245,7 +280,10 @@ R_API void r_str_case(char *str, bool up) {
 }
 
 R_API R_MUSTUSE char *r_str_r2_prefix(const char *str) {
-	return r_str_newf ("%s%s%s", r_sys_prefix (NULL), R_SYS_DIR, str);
+	char *pfx = r_sys_prefix (NULL);
+	char *res = r_str_newf ("%s%s%s", pfx, R_SYS_DIR, str);
+	free (pfx);
+	return res;
 }
 
 // Compute a modified 64 bit DJB hash of a string
@@ -997,6 +1035,7 @@ R_API R_MUSTUSE char *r_str_replace_icase(char *str, const char *key, const char
 			if (vlen > klen) {
 				newstr = realloc (str, slen + 1);
 				if (!newstr) {
+					free (str);
 					return NULL;
 				}
 				str = newstr;
@@ -1925,7 +1964,7 @@ R_API size_t r_str_ansi_strip(char *str) {
 #if 0
 // unused
 // insert a string into another string, supports ansi control chars
-R_API char *r_str_insert(R_OWN char *src, int pos, const char *str) {
+R_API char *r_str_insert(R_OWNED char *src, int pos, const char *str) {
 	r_str_var (a, pos, src);
 	char *b = strdup (src + pos + r_str_ansi_len (str));
 	char *r = r_str_newf ("%s%s%s", a, str, b);
@@ -4005,23 +4044,25 @@ R_API char *r_str_nextword(char *s, char ch) {
 	return p;
 }
 
-R_API char *r_str_scale(const char *s, int w, int h) {
+R_API char *r_str_scale(const char *s, const int w, const int h) {
 	R_RETURN_VAL_IF_FAIL (s && w > 0 && h > 0, NULL);
 	char *str = strdup (s);
 	RList *lines = r_str_split_list (str, "\n", 0);
 	int i, j;
-	int rows = r_list_length (lines);
+	const int rows = r_list_length (lines);
 	RList *out = r_list_newf (free);
 
 	int curline = -1;
 	char *linetext = (char *)r_str_pad (NULL, 0, ' ', w);
 	for (i = 0; i < h; i++) {
-		int zoomedline = (int) (i *((double)rows / h));
+		const int zoomedline = (int) (i *((double)rows / h));
 		const char *srcline = r_list_get_n (lines, zoomedline);
-		int cols = strlen (srcline);
-		for (j = 0; j < w; j++) {
-			int zoomedcol = (int) (j *((double)cols / w));
-			linetext[j] = srcline[zoomedcol];
+		const int cols = srcline ? strlen (srcline) : 0;
+		if (cols > 0) {
+			for (j = 0; j < w; j++) {
+				const int zoomedcol = (int) (j *((double)cols / w));
+				linetext[j] = srcline[zoomedcol];
+			}
 		}
 		if (curline != zoomedline) {
 			r_list_append (out, strdup (linetext));
@@ -4067,6 +4108,7 @@ R_API const char *r_str_str_xy(const char *s, const char *word, const char *prev
 // version.c
 #include <r_userconf.h>
 #include <r_util.h>
+#include <r_lib.h>
 
 #ifndef R2_GITTAP
 #define R2_GITTAP ""
@@ -4112,8 +4154,8 @@ R_API char *r_str_version(const char *program) {
 #if WITH_GPL
 	gplstr = " gpl";
 #endif
-	char *s = r_str_newf ("%s " R2_VERSION " %d @ " R_SYS_OS "-" R_SYS_ARCH "-%d\n",
-		program, R2_VERSION_COMMIT,
+	char *s = r_str_newf ("%s " R2_VERSION " +%d abi:%d @ " R_SYS_OS "-" R_SYS_ARCH "-%d\n",
+		program, R2_VERSION_COMMIT, R2_ABIVERSION,
 		(R_SYS_BITS_CHECK (R_SYS_BITS, 64))? 64: 32);
 	int csv = R2_CSVERSION;
 	s = r_str_appendf (s, "birth: git.%s " R2_BIRTH "\n",
